/*--
 Copyright (C) 2005 Tim Solley.
 All rights reserved.

 Redistribution and use in source and binary forms, with or without
 modification, are permitted provided that the following conditions
 are met:

 1. Redistributions of source code must retain the above copyright
 notice, this list of conditions, and the following disclaimer.

 2. Redistributions in binary form must reproduce the above copyright
 notice, this list of conditions, and the disclaimer that follows
 these conditions in the documentation and/or other materials
 provided with the distribution.

 3. The name "Deadbolt" may be used to endorse or promote products
 derived from this software without prior written permission.

 4. Products derived from this software may not be called "Deadbolt", nor
 may "Deadbolt" appear in their name, without prior written permission
 from the Deadbolt Project Management timsolley@yahoo.com.

 In addition, we request (but do not require) that you include in the
 end-user documentation provided with the redistribution and/or in the
 software itself an acknowledgement equivalent to the following:
 "This product includes software developed by the
 Deadbolt Project (http://deadbolt.sourceforge.net/)."
 Alternatively, the acknowledgment may be graphical using the logos
 available at http://deadbolt.sourceforge.net.

 THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 DISCLAIMED.  IN NO EVENT SHALL THE DEADBOLT AUTHORS OR THE PROJECT
 CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 SUCH DAMAGE.

 This software consists of voluntary contributions made by many
 individuals on behalf of the Deadbolt Project and was originally
 created by Tim Solley timsolley@yahoo.com.  For more information
 on the Deadbolt Project, please see <http://deadbolt.sourceforge.net/>.
 */

package net.sf.deadbolt;

import java.io.File;
import java.io.IOException;
import java.util.*;

import javax.servlet.*;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import net.sf.deadbolt.handlers.DeadboltHandler;
import net.sf.deadbolt.model.Room;
import net.sf.deadbolt.model.URLMapping;
import net.sf.deadbolt.tags.DisplayErrorsTag;

import org.apache.commons.lang.StringUtils;
import org.apache.log4j.Logger;
import org.jdom.Document;
import org.jdom.Element;
import org.jdom.JDOMException;
import org.jdom.input.SAXBuilder;


/**
 * This class is the meat of the Deadbolt framework. This filter is executed on
 * every request for an application, and it determines the authentication and
 * authorization setup for the requested resource.
 * 
 * @author Tim Solley <timsolley@yahoo.com>
 */
public class DeadboltFilter implements Filter {
    /* Jakarta Commons Logging is used here */
    private static Logger logger = Logger.getLogger(DeadboltFilter.class.getName());

    /* Added to the object in case a handler needs access to the ServletContext */
    private static ServletContext context;

    /*
     * This is a list of mappings from URL pattern to a room. This indicates the
     * room to be used when authenticating a resource.
     */
    private static List urlMappings;

    /*
     * The list of error messages that will be shown to the user if any are
     * added.
     */
    private static Map errorMessages;
    
    /*
     * This is the Room to contain global information.
     */
    private static Room globalRoom;

    /**
     * This method will return the <code>Map</code> of error messages, which
     * are <code>String</code> objects. This is used by the
     * {@link DeadboltHandler}to put messages in the request for later display
     * by the {@link DisplayErrorsTag}custom tag on the JSP page.
     * 
     * @return Returns the errorMessages.
     */
    public static Map getErrorMessages() {
        return errorMessages;
    }

    /**
     * This method is the starting point the the Deadbolt enabled application.
     * It processes the configuration XML file and stores the configuration
     * information in memory when the application starts up. That information
     * will be later used by the <code>doFilter</code> method to secure the
     * requested resource.
     */
    public void init(FilterConfig config) throws ServletException {
        logger.debug("ENTERING: init");

        context = config.getServletContext();
        /*
         * Get the config file: The config file is specified in the web.xml file
         * under an init-param named "config-file" for the filter. We simply use
         * the application context to the get the path within the application.
         * If the user wants to put the file outside the application, this part
         * will not work. This is a good area for someone to make that more
         * flexible, allowing a user to put the file anywhere, taking the
         * configuration information out of the application and allowing it to
         * be modified without having to create another build.
         */
        File configFile = new File(context.getRealPath(config
                .getInitParameter("config-file")));

        /*
         * Read the XML and do setup work: JDOM is used to process the config
         * file. This section takes the XML file in pieces, processing each
         * block sequentially.
         */
        SAXBuilder builder = new SAXBuilder();
        builder.setValidation(true);
        builder.setFeature("http://apache.org/xml/features/validation/schema", true);
        urlMappings = new ArrayList();

        try {
            Document document = builder.build(configFile);
            Element deadboltConfigElement = document.getRootElement();

            // Process the error messages, starting with the global error page
            String globalErrorPage = deadboltConfigElement
                    .getChildText("global-error-page");

            /*
             * Process the handlers: Handlers can be reused within the config
             * file. So, to avoid redundency, the sets up the unique list of
             * them. They then just reference the handler name in the room. Here
             * we just process that list, and store them in memory so that they
             * can be accessed when processing rooms.
             */
            List handlerList = deadboltConfigElement.getChildren("handler");
            Map handlers = new Hashtable();
            for (Iterator it = handlerList.iterator(); it.hasNext();) {
                Element handlerElement = (Element) it.next();
                handlers.put(handlerElement.getChildText("handler-name"),
                        handlerElement.getChildText("handler-class"));
            }
            
            /*
             * Process the global handlers.  These will be run on every request
             * that Deadbolt responds to, before control is passed off to the
             * Rooms for processing.
             */
            List globalHandlersList = deadboltConfigElement.getChildren("global-handler");
            if(globalHandlersList.size() != 0) {
                globalRoom = new Room();
                TreeMap globalHandlerMap = new TreeMap();
                Map globalInitParams = new Hashtable();
                for(Iterator iterator = globalHandlersList.iterator(); iterator.hasNext();) {
                    Element handler = (Element) iterator.next();
                    String handlerClass = (String) handlers.get(handler.getChildText("handler-name"));
                    globalHandlerMap.put(handler.getChildText("execution-order"), handlerClass);
                    logger.debug("Global handler: " + handler.getChildText("execution-order") + " | " +
                            handlerClass);
                    
                    /*
                     * Set up the initial parameters for this global handler
                     */
                    List initParamsElements = handler.getChildren("init-param");
                    for (Iterator i = initParamsElements.iterator(); i.hasNext();) {
                        Element initParam = (Element) i.next();
                        // Add the current param to the list
                        globalInitParams.put(initParam.getChildText("param-name"),
                                initParam.getChildText("param-value"));
                        logger.debug("Param name: " + initParam.getChildText("param-name"));
                        logger.debug("Param value: " + initParam.getChildText("param-value"));
                    }
                }
                globalRoom.setInitParams(globalInitParams);
                globalRoom.setHandler(globalHandlerMap);
            }
            

            /*
             * Process the rooms.
             */
            List roomList = deadboltConfigElement.getChildren("room");
            for (Iterator it = roomList.iterator(); it.hasNext();) {
                // Set up a new room
                Room room = new Room();
                Map initParams = new Hashtable();
                TreeMap handlerMap = new TreeMap();
                Element roomElement = (Element) it.next();

                /*
                 * Set up the handler chain for this room: The user can specify
                 * multiple handlers for a room. When they do this, they will be
                 * executed sequentially when a request for the specified
                 * resource comes in. The user specifies the execution order
                 * using the "execution-order" element. This section reads in
                 * the handler, and adds it to a TreeMap, which will
                 * automatically put it in the correct order based on the
                 * "execution-order".
                 */
                List handlerListElement = roomElement.getChildren("handler");
                for (Iterator iter = handlerListElement.iterator(); iter
                        .hasNext();) {
                    Element handler = (Element) iter.next();
                    String handlerClass = (String) handlers.get(handler
                            .getChildText("handler-name"));
                    // Add the handler to the TreeMap, going into the correct
                    // order
                    handlerMap.put(handler.getChildText("execution-order"),
                            handlerClass);
                    logger.debug("Handler: "
                            + handler.getChildText("execution-order") + " | "
                            + handlerClass);
                }
                // Add the handler chain to the room
                room.setHandler(handlerMap);

                /*
                 * Set up the init parameters for this room: Init params can be
                 * defined by the user. They can have as many as they want.
                 * These will be available to the custom handler by way of the
                 * room.
                 */
                List initParamsElements = roomElement.getChildren("init-param");
                for (Iterator i = initParamsElements.iterator(); i.hasNext();) {
                    Element initParam = (Element) i.next();
                    // Add the current param to the list
                    initParams.put(initParam.getChildText("param-name"),
                            initParam.getChildText("param-value"));
                    logger.debug("Param name: " + initParam.getChildText("param-name"));
                    logger.debug("Param value: " + initParam.getChildText("param-value"));
                }
                // Add the init params to the room
                room.setInitParams(initParams);

                /*
                 * Set up the error page for this room: If no error page is
                 * specified, then the global error page will be used.
                 */
                String errorPage = roomElement.getChildText("error-page");
                if (errorPage == null) {
                    room.setErrorPage(globalErrorPage);
                } else {
                    room.setErrorPage(errorPage);
                }

                /*
                 * Set up the URL mappings for this room: This will be the
                 * mechanism that matches a URL pattern to a room.
                 */
                List urls = roomElement.getChildren("url-pattern");
                for (Iterator i = urls.iterator(); i.hasNext();) {
                    URLMapping mapping = new URLMapping();
                    mapping.setRoom(room);
                    // Get the URL pattern from the config file
                    String pattern = ((Element) i.next()).getText();
                    /*
                     * The requested pattern will be tested against this pattern
                     * later on in the doFilter method using a regular
                     * expression. To do this, we need to ensure that any
                     * asterisks entered by the user are translated to a regular
                     * express character. So we replace stars with ".++", for
                     * zero to infinity of any character.
                     */
                    pattern = StringUtils.replace(pattern, "*", ".++");
                    mapping.setUrlPattern(pattern);
                    // Add this mapping to the list of mappings
                    urlMappings.add(mapping);
                }
            }

            /*
             * Set up the list of error message keys: These keys are used by the
             * user in the handlers to send error messages to the client using
             * consistent verbiage. This also enables the user to make spelling,
             * grammatical, or wording changes in one place, outside code,
             * without having to recompile the code.
             */
            errorMessages = new Hashtable();
            Element errorMessagesElement = deadboltConfigElement
                    .getChild("error-messages");
            List errorMessageList = errorMessagesElement
                    .getChildren("error-message");
            for (Iterator it = errorMessageList.iterator(); it.hasNext();) {
                Element errorMessage = (Element) it.next();
                errorMessages.put(errorMessage.getChildText("message-key"),
                        errorMessage.getChildText("message-content"));
            }
        } catch (IOException ioe) {
            logger.warn("An IOException occurred during init: "
                            + ioe.getMessage());
        } catch (JDOMException jde) {
            logger.warn("A JDOM Exception occurred during init: " + jde.getMessage());
        }
        logger.debug("EXITING: init");
    }

    /**
     * This method will be called on each request to the application, provided
     * the user used <code>/*</code> as the url-pattern element in the web.xml
     * file. This method will examine the request, compare it's URL to a list of
     * URL patterns defined in the config file, and execute handlers based on
     * any room found that matches the URL pattern. It will then let the user
     * pass to the requested resource, or send the user to the specified error
     * page if not allowed. If any handler sends back a false, the user will not
     * be permitted to enter.
     * 
     * It should be noted that if a resource exists, and it's URL pattern is not
     * specified in the config file, and a blanket <code>/*</code> is not
     * used, a client will be allowed to access the resource and will not be
     * protected by Deadbolt.
     */
    public void doFilter(ServletRequest request, ServletResponse response,
            FilterChain chain) throws IOException, ServletException {
        logger.debug("ENTERING: doFilter");
        // Cast the request and response so HTTP specific methods can be used
        HttpServletRequest httpRequest = (HttpServletRequest) request;
        HttpServletResponse httpResponse = (HttpServletResponse) response;
        Room room = null;

        // This is a flag to specify if the request will be allowed in or not
        boolean forwardRequest = false;
        
        /*
         * Before we execute any URL specific Rooms, check for a global Room
         * and execute it first.
         */
        if(globalRoom != null) {
            try {
                DeadboltHandler handler = null;
                Collection handlers = globalRoom.getHandlers().values();
                for(Iterator handlerIterator = handlers.iterator();
                    handlerIterator.hasNext();) {
                    String handlerClass = (String) handlerIterator.next();
                    logger.debug("Executing global handler: " + handlerClass);
                    /*
                     * Since we don't know the class of the actual handler,
                     * we're using the DeadboltHandler class, which all handlers
                     * extend. We instantiate the actual class using the String
                     * name.
                     */
                    handler = (DeadboltHandler) Class.forName(handlerClass)
                            .newInstance();
                    
                    // Here's the meat. Do the work on this handler.
                    forwardRequest = handler.authenticate(httpRequest,
                            httpResponse, globalRoom);
                    logger.debug("Global handler returned: " + forwardRequest);
                    if (!forwardRequest)
                        break;
                }
            } catch (InstantiationException ie) {
                logger.warn(ie);
                throw new ServletException(ie); 
            } catch (IllegalAccessException iae) {
                logger.warn(iae);
                throw new ServletException(iae);
            } catch (ClassNotFoundException cnfe) {
                logger.warn(cnfe);
                throw new ServletException(cnfe);
            }
        }
        
        // This is the URL pattern of the request that just came in
        String currentPattern = httpRequest.getServletPath();
        // This is a flag to identify if a matching room is found
        boolean matchFound = false;

        /*
         * Iterate over the mappings to find the handlers to execute on this
         * request On each pass, compare the URL pattern in the mapping to that
         * of the request, and set the matchFound flag to true and set the room
         * if something is found.
         */
        for (Iterator it = urlMappings.iterator(); it.hasNext();) {
            URLMapping mapping = (URLMapping) it.next();
            // Use a regular expression to compare the URL for the request to
            // the URL in the mapping
            if (currentPattern.matches(mapping.getUrlPattern())) {
                matchFound = true;
                // Set the room to be used below
                room = mapping.getRoom();
                break;
            }
        }

        // If a match was found, execute the handlers for that room
        if(!matchFound) {
            forwardRequest = true;
        } else if(forwardRequest & room != null) {
            try {
                DeadboltHandler handler = null;
                // Get a Collection, because TreeMap has no Iterator or the like
                Collection handlers = room.getHandlers().values();
                /*
                 * Iterate over the handler chain for this room, calling the
                 * authenticate method on each one. If any handler returns a
                 * false, then set forwardRequest to false and exit the loop.
                 */
                for (Iterator i = handlers.iterator(); i.hasNext();) {
                    String handlerClass = (String) i.next();
                    logger.debug("Executing handler: " + handlerClass);
                    /*
                     * Since we don't know the class of the actual handler,
                     * we're using the DeadboltHandler class, which all handlers
                     * extend. We instantiate the actual class using the String
                     * name.
                     */
                    handler = (DeadboltHandler) Class.forName(handlerClass)
                            .newInstance();

                    // Here's the meat. Do the work on this handler.
                    forwardRequest = handler.authenticate(httpRequest,
                            httpResponse, room);
                    logger.debug("Handler returned: " + forwardRequest);
                    if (!forwardRequest)
                        break;
                }
            } catch (InstantiationException ie) {
                logger.warn(ie);
                throw new ServletException(ie); 
            } catch (IllegalAccessException iae) {
                logger.warn(iae);
                throw new ServletException(iae);
            } catch (ClassNotFoundException cnfe) {
                logger.warn(cnfe);
                throw new ServletException(cnfe);
            }
        }

        /*
         * If the user is authenticated and authorized, send the request off to
         * the resource. Otherwise, send the user to the error page defined for
         * the room.
         */
        if (forwardRequest) {
            logger.debug("The request was approved.  Continuing to resource.");
            chain.doFilter(request, response);
        } else {
            logger.debug("The request was denied.  Forwarding to error page.");
            if (room.getErrorPage() != null) {
                RequestDispatcher dispatcher = httpRequest
                        .getRequestDispatcher(room.getErrorPage());
                dispatcher.forward(request, response);
            }
        }
        logger.debug("EXITING: doFilter");
    }

    /**
     * This is here for compliance with the <code>Filter</code> interface.
     * Nothing is done in this method.
     */
    public void destroy() {
    }
}