/*--
 Copyright (C) 2005 Tim Solley.
 All rights reserved.

 Redistribution and use in source and binary forms, with or without
 modification, are permitted provided that the following conditions
 are met:

 1. Redistributions of source code must retain the above copyright
 notice, this list of conditions, and the following disclaimer.

 2. Redistributions in binary form must reproduce the above copyright
 notice, this list of conditions, and the disclaimer that follows
 these conditions in the documentation and/or other materials
 provided with the distribution.

 3. The name "Deadbolt" may be used to endorse or promote products
 derived from this software without prior written permission.

 4. Products derived from this software may not be called "Deadbolt", nor
 may "Deadbolt" appear in their name, without prior written permission
 from the Deadbolt Project Management timsolley@yahoo.com.

 In addition, we request (but do not require) that you include in the
 end-user documentation provided with the redistribution and/or in the
 software itself an acknowledgement equivalent to the following:
 "This product includes software developed by the
 Deadbolt Project (http://deadbolt.sourceforge.net/)."
 Alternatively, the acknowledgment may be graphical using the logos
 available at http://deadbolt.sourceforge.net.

 THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 DISCLAIMED.  IN NO EVENT SHALL THE DEADBOLT AUTHORS OR THE PROJECT
 CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 SUCH DAMAGE.

 This software consists of voluntary contributions made by many
 individuals on behalf of the Deadbolt Project and was originally
 created by Tim Solley timsolley@yahoo.com.  For more information
 on the Deadbolt Project, please see <http://deadbolt.sourceforge.net/>.
 */

package net.sf.deadbolt.handlers;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import net.sf.deadbolt.DeadboltEnvironment;
import net.sf.deadbolt.DeadboltFilter;
import net.sf.deadbolt.model.Room;

import org.apache.log4j.Logger;

/**
 * This is the base class that all handlers extend from. The handler that you
 * write only needs to override the <code>authenticate</code> method that will
 * be called when a resource is requested that is protected by a room with your
 * handler defined.
 * 
 * @author Tim Solley <timsolley@yahoo.com>
 */
public abstract class DeadboltHandler {
    private static Logger logger = Logger.getLogger(DeadboltHandler.class.getName());

    /**
     * This method is the main body of the handler, which will tell the
     * framework whether to let the user in or not. You can do whatever you like
     * in this method, such as checking a user's digital certificate, checking
     * LDAP, checking user information stored in the session, or even checking a
     * database. Other uses might be for logging or statistical purposes.
     * 
     * Handlers can be chained in Deadbolt, as specified in the
     * deadbolt-config.xml file. It's important to remember that if you're
     * chaining handlers, and one handler depends on information obtained in the
     * previous handler, that the previous handler should put this information
     * in a well known place, such as in the HttpServletRequest.
     * 
     * @param request
     * @param response
     * @return Whether to let the user pass this handler or not.
     */
    public abstract boolean authenticate(HttpServletRequest request,
            HttpServletResponse response, Room room);

    /**
     * This method will add an error to the request, which can later be used by
     * the <code>DisplayErrorsTag</code> custom tag in a JSP error page.
     * 
     * In a user defined handler, just call this method, passing in the request
     * and a error key, as defined in the deadbolt-config.xml file. This will
     * put the actual text of the error message in the request for the JSP page.
     * 
     * It's important for a developer writing a handler to make sure that the
     * spelling of the error key is correct. If not, then no message will be
     * found.
     * 
     * @param request
     * @param errorKey
     */
    public void addErrorKey(HttpServletRequest request, String errorKey) {
        logger.info("ENTERING: addError");
        // Get the collection of error messages that pertain to the application
        Map errors = DeadboltFilter.getErrorMessages();
        // Get the actual error message based on the key
        String errorMessage = (String) errors.get(errorKey);
        if (errorMessage == null) {
            logger.warn("The error key: " + errorKey
                    + " was not found!  Check your deadbolt-config.xml file.");
        } else {
            addErrorMessage(request, errorMessage);
        }
        logger.info("EXITING: addError");
    }

    /**
     * This method does the same thing as the <code>addError</code> method,
     * but takes in a <code>List</code> as a parameter. This <code>List</code>
     * must contain only <code>String</code> objects.
     * 
     * @param request
     * @param errors
     *            A List of errors
     */
    public void addErrors(HttpServletRequest request, List errors) {
        logger.info("ENTERING: addErrors");

        if (request.getAttribute(DeadboltEnvironment.GLOBAL_ERROR_KEY) == null) {
            request.setAttribute(DeadboltEnvironment.GLOBAL_ERROR_KEY,
                    new ArrayList());
        }
        ((List) request.getAttribute(DeadboltEnvironment.GLOBAL_ERROR_KEY))
                .addAll(errors);

        logger.info("EXITING: addErrors");
    }
    
    /**
     * This method adds an error message without the need for a key.  This is
     * mostly used internally for one off messages.
     * 
     * @param request
     * @param errorMessage
     */
    public void addErrorMessage(HttpServletRequest request, String errorMessage) {
        /*
         * If this is the first time an error has been added during this
         * request, we'll need to create an attribute in the request object. *
         */
        if (request.getAttribute(DeadboltEnvironment.GLOBAL_ERROR_KEY) == null) {
            request.setAttribute(DeadboltEnvironment.GLOBAL_ERROR_KEY,
                    new ArrayList());
        }
        // Add the error to the list in the request
        ((List) request.getAttribute(DeadboltEnvironment.GLOBAL_ERROR_KEY))
                .add(errorMessage);
    }
}