/*--
Copyright (C) 2005 Tim Solley.
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions
are met:

1. Redistributions of source code must retain the above copyright
notice, this list of conditions, and the following disclaimer.

2. Redistributions in binary form must reproduce the above copyright
notice, this list of conditions, and the disclaimer that follows
these conditions in the documentation and/or other materials
provided with the distribution.

3. The name "Deadbolt" may be used to endorse or promote products
derived from this software without prior written permission.

4. Products derived from this software may not be called "Deadbolt", nor
may "Deadbolt" appear in their name, without prior written permission
from the Deadbolt Project Management timsolley@yahoo.com.

In addition, we request (but do not require) that you include in the
end-user documentation provided with the redistribution and/or in the
software itself an acknowledgement equivalent to the following:
"This product includes software developed by the
Deadbolt Project (http://deadbolt.sourceforge.net/)."
Alternatively, the acknowledgment may be graphical using the logos
available at http://deadbolt.sourceforge.net.

THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
DISCLAIMED.  IN NO EVENT SHALL THE DEADBOLT AUTHORS OR THE PROJECT
CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
SUCH DAMAGE.

This software consists of voluntary contributions made by many
individuals on behalf of the Deadbolt Project and was originally
created by Tim Solley timsolley@yahoo.com.  For more information
on the Deadbolt Project, please see <http://deadbolt.sourceforge.net/>.
*/
package net.sf.deadbolt.handlers;

import java.util.Arrays;
import java.util.List;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import net.sf.deadbolt.model.Room;

import org.apache.log4j.Logger;

/**
 * This class is used to restrict or allow access to resources based on the user's
 * IP address.
 * 
 * @author Tim Solley <timsolley@yahoo.com>
 */
public class IPFilterHandler extends DeadboltHandler {
    private static Logger logger = Logger.getLogger(IPFilterHandler.class.getName());
    
    /**
     * This is the standard Deadbolt authenticate method.  It will reject or allow
     * access to resources based on what parameter and addresses are specified in
     * the Deadbolt descriptor.
     */
    public boolean authenticate(HttpServletRequest request,
            HttpServletResponse response, Room room) {
        logger.debug("ENTERING: authenticate");
        
        // Get the IP address of the client
        String remoteAddress = request.getRemoteAddr();
        logger.debug("Remote IP address: " + remoteAddress);
        
        // Get the host name of of the client
        String remoteHost = request.getRemoteHost();
        logger.debug("Remote host: " + remoteAddress);
        
        // Check for the existence of both params, and let the developer know
        // that only one should be used
        if(room.initParamExists("EXCLUDED-LIST") && room.initParamExists("INCLUDED-LIST")) {
            logger.warn("Either EXCLUDED-LIST or INCLUDED-LIST should be specified, not both.  Deadbolt will default to using the EXCLUDED-LIST only.");
        }
        
        // Get the list of excluded addresses
        if(room.initParamExists("EXCLUDED-LIST")) {
            logger.debug("EXCLUDED-LIST was specified.");
            String excludedAddressesParam = room.getInitParam("EXCLUDED-LIST");
            List excludedAddresses;
            String[] excludedAddressesArray = excludedAddressesParam.split(",");
            for(int i = 0; i < excludedAddressesArray.length; i++) {
                excludedAddressesArray[i] = excludedAddressesArray[i].trim();
            }
            excludedAddresses = Arrays.asList(excludedAddressesArray);
            
            if(excludedAddresses.contains(remoteAddress) | excludedAddresses.contains(remoteHost)) {
                logger.debug("The address is in the excluded list.  Return false.");
                addErrorKey(request, "ip.filter.handler.restricted");
                return false;
            } else {
                logger.debug("The address is not in the excluded list.  Return true.");
                return true;
            }
        }
        
        // Get the list of included addresses
        if(room.initParamExists("INCLUDED-LIST")) {
            logger.debug("INCLUDED-LIST was specified.");
            String includedAddressesParam = room.getInitParam("INCLUDED-LIST");
            List includedAddresses;
            String[] includedAddressesArray = includedAddressesParam.split(",");
            for(int i = 0; i < includedAddressesArray.length; i++) {
                includedAddressesArray[i] = includedAddressesArray[i].trim();
            }
            includedAddresses = Arrays.asList(includedAddressesArray);
        
            if(includedAddresses.contains(remoteAddress) | includedAddresses.contains(remoteHost)) {
                logger.debug("The address is in the included list.  Return true.");
                return true;
            } else {
                logger.debug("The address is not in the included list.  Return false.");
                addErrorKey(request, "ip.filter.handler.restricted");
                return false;
            }
        }
        
        logger.debug("EXITING: authenticate");
        return false;
    }

}
