/*--
Copyright (C) 2005 Tim Solley.
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions
are met:

1. Redistributions of source code must retain the above copyright
notice, this list of conditions, and the following disclaimer.

2. Redistributions in binary form must reproduce the above copyright
notice, this list of conditions, and the disclaimer that follows
these conditions in the documentation and/or other materials
provided with the distribution.

3. The name "Deadbolt" may be used to endorse or promote products
derived from this software without prior written permission.

4. Products derived from this software may not be called "Deadbolt", nor
may "Deadbolt" appear in their name, without prior written permission
from the Deadbolt Project Management timsolley@yahoo.com.

In addition, we request (but do not require) that you include in the
end-user documentation provided with the redistribution and/or in the
software itself an acknowledgement equivalent to the following:
"This product includes software developed by the
Deadbolt Project (http://deadbolt.sourceforge.net/)."
Alternatively, the acknowledgment may be graphical using the logos
available at http://deadbolt.sourceforge.net.

THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
DISCLAIMED.  IN NO EVENT SHALL THE DEADBOLT AUTHORS OR THE PROJECT
CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
SUCH DAMAGE.

This software consists of voluntary contributions made by many
individuals on behalf of the Deadbolt Project and was originally
created by Tim Solley timsolley@yahoo.com.  For more information
on the Deadbolt Project, please see <http://deadbolt.sourceforge.net/>.
*/
package net.sf.deadbolt.handlers;



import java.util.*;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import net.sf.deadbolt.model.Room;

import org.apache.log4j.Logger;

/**
 * This class will secure a Room from SQL injection attacks.
 * 
 * @author Tim Solley <timsolley@yahoo.com>
 */
public class SQLInjectionHandler extends DeadboltHandler {
       private static Logger logger = Logger.getLogger(SQLInjectionHandler.class.getName());
       
       /**
        * These are all the keywords that this handler will check for
        */
       private String[] badStringsWeak = new String[] {"'", ";", "--"};
       
       private String[] badStringsStrong = new String[] {"'", ";", "--", "union", "drop", "insert",
               "update", "delete", "having", "group by", "select", "sum", "max", "min",
               "values", "@@", "from", "where", "create", "begin", "declare", "end",
               "exec", "shutdown", "xp_", "master", "bulk insert"};
       private String[] badStrings;
       
       private List excludedFields;
       
      /**
       * This method runs through the parameters in the request and checks them
       * for the keywords that are not allowed.  
       */
      public boolean authenticate(HttpServletRequest request, 
              HttpServletResponse response, Room room) {
          logger.debug("ENTERING: authenticate");
          
          boolean result = true;
          
          /* Check for the existence of the "LEVEL" init param specifying the 
           * security level from this room.  If the level is strong, we'll check
           * for an expanded set of keywords.  This is primarily for large
           * text fields and the like where some of the strong keywords would
           * be valid.
           */
          if("WEAK".equals(room.getInitParam("LEVEL"))) {
              badStrings = badStringsWeak;
          } else {
              badStrings = badStringsStrong;
          }
          
          // Check for any excluded fields
          String excludedFieldsParam = room.getInitParam("EXCLUDED-FIELDS");
          logger.debug("The following fields will be excluded from this handler: " + excludedFieldsParam);
          if(excludedFieldsParam != null) {
              String[] excludedFieldsArray = excludedFieldsParam.split(",");
              for(int i = 0; i < excludedFieldsArray.length; i++) {
                  excludedFieldsArray[i] = excludedFieldsArray[i].trim();
              }
              excludedFields = Arrays.asList(excludedFieldsArray);
          }
          else
              excludedFields = new ArrayList(0);

          for(Enumeration parameterEnumeration = request.getParameterNames();
                  parameterEnumeration.hasMoreElements();) {
              String element = (String) parameterEnumeration.nextElement();
              if(!excludedFields.contains(element)) {
                  logger.debug("The following parameter is being tested: " + element);
                  result = testValue(request.getParameter(element), request, room);
              }
              
              if(!result) {
                  break;
              }
          }
          
          logger.debug("EXITING: authenticate");
          return result;
      }
           
      /**
       * This method scans the parameter value sent in, checking for the 
       * prohibited strings.
       * 
       * @param value
       * @param request
       * @param room
       * @return
       */
      private boolean testValue(String value, HttpServletRequest request, Room room) {
          logger.debug("ENTERING: testValue");
          boolean result = true;
          // Loop over the array of bad strings and check each one against the
          // current request parameter          
          for(int key = 0; key < badStrings.length; key++) {
              logger.debug("Testing value = " + value + " and badString = " + badStrings[key]);
              if(value.matches(badStrings[key])) {
                  logger.debug("A forbidden string was found: " + value);
                  addErrorMessage(request, (String) room.getInitParam("ErrorMessage"));
                  logger.debug("The value was rejected, returning false");
                  result = false;
                  break;
              }
          }

          logger.debug("EXITING: testValue");
          return result;
      }
       
}
